#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Max Thomas
Load in firn data, age data, and Zuiderweg et al 2013 data
Produce loess over NEEM 2008 and 2009 firn profiles
Plot all firn profiles plus loess
pickle data for master plot script
"""

# import packages
import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
import plotnine as p9

# Options for user
Nitr    = 200    # iterations for monte carlo
npoints = 25    # number of points at which to evaluate loess
xmin11    = 1951.6  # start year for loess prediction
xmax11    = 2009.55  # end year for loess prediction
xmin12    = 1953.6  # start year for loess prediction
xmax12    = 2009.55  # end year for loess prediction
xmin113   = 1973  # start year for loess prediction
xmax113   = 2009.55  # end year for loess prediction
age_def = 1     # age plotting: 0->plot median, 1->plot mean, 2->plot mode

# Data sources
data_sheets1 = '../data_summary.xls'
data_sheets2 = '../PM_model_output/NEEM09'
data_sheets3 = '../PM_model_output/NEEM08'

##############################################################################
##############################################################################
# Define functions
##############################################################################
##############################################################################

# ages_delta loads in ages and puts NEEM 2008 and 2009 in the same data frame
def load_ages(cfc_n):
    ages2009 = '/age_zmes_f' + str(cfc_n) + '_dall_ew1_09.txt'
    ages2008 = '/age_zmes_d13f' + str(cfc_n) + '_dall_ew1_eu.txt'
    age_out09 = pd.read_csv(data_sheets2+ages2009,\
                         sep="\s+",header=1,names=['age','0.0','10.5','20.4','30.2','39.2','50.7','60.3','62.0','63.8','66.8','69.4','71.9','junk'])
    age_out08 = pd.read_csv(data_sheets3+ages2008,\
                         sep="\s+",header=1,names=['age','0.0','5.0','10.1','14.9','20.3','29.7','39.3','48.6','60.0'])
                         #sep="\s+",header=1,names=['age','0.0','2.6','5.0','7.6','10.1','14.9','19.8','27.6','34.8','42.4','50.0','55.0','57.4','60.0','62.0','63.8','65.8','68.0','70.0','72.0','74.0','76.0','77.8'])
    age_out = {'NEEM08':age_out08,'NEEM09':age_out09}
    return age_out

# method2 builds the data set for the loess
def method2(cfc_n,firn0Trop1,VPDB,XSA,YSA):
    if cfc_n==12:
        for ir in CFC12.index:
            dstr = CFC12['depth'][ir]
            if CFC12['campaign'][ir]<1:               
                age_add = np.random.choice(2008.55 - age12['NEEM08']['age'],1,\
                              p=age12['NEEM08'][dstr])
            elif CFC12['campaign'][ir]==1:
                age_add = np.random.choice(2009.55 - age12['NEEM09']['age'],1,\
                              p=age12['NEEM09'][dstr])
                  
            XSA = np.append(XSA,age_add)
            if firn0Trop1==0:
                if VPDB==0:
                    YSA = np.append(YSA,CFC12['del_firn'][ir])
                elif VPDB==1:
                    YSA = np.append(YSA,CFC12['del_firn_vpdb'][ir])
            elif firn0Trop1==1:
                if VPDB==0:
                    YSA = np.append(YSA,CFC12['del_firn'][ir])
                elif VPDB==1:
                    YSA = np.append(YSA,CFC12['del_trop_vpdb'][ir])
    elif cfc_n==11:
        for ir in CFC11.index:
            dstr = CFC11['depth'][ir]
            if CFC11['campaign'][ir]<1:               
                age_add = np.random.choice(2008.55 - age11['NEEM08']['age'],1,\
                              p=age11['NEEM08'][dstr])
            elif CFC11['campaign'][ir]==1:
                age_add = np.random.choice(2009.55 - age11['NEEM09']['age'],1,\
                              p=age11['NEEM09'][dstr])
                  
            XSA = np.append(XSA,age_add)
            if firn0Trop1==0:
                YSA = np.append(YSA,CFC11['del_firn'][ir])
            elif firn0Trop1==1:
                YSA = np.append(YSA,CFC11['del_trop'][ir])
    elif cfc_n==113:
        for ir in CFC113.index:
            dstr = CFC113['depth'][ir]
            if CFC113['campaign'][ir]<1:               
                age_add = np.random.choice(2008.55 - age113['NEEM08']['age'],1,\
                              p=age113['NEEM08'][dstr])
            elif CFC11['campaign'][ir]==1:
                age_add = np.random.choice(2009.55 - age113['NEEM09']['age'],1,\
                              p=age113['NEEM09'][dstr])
                  
            XSA = np.append(XSA,age_add)
            if firn0Trop1==0:
                YSA = np.append(YSA,CFC113['del_firn'][ir])
            elif firn0Trop1==1:
                YSA = np.append(YSA,CFC113['del_trop'][ir])
    return XSA, YSA


# stats2 calculates the loess
lw_model = p9.stat_smooth()
xsel11 = pd.DataFrame({'x':np.linspace(xmin11,xmax11,npoints)})
xsel12 = pd.DataFrame({'x':np.linspace(xmin12,xmax12,npoints)})
xsel113 = pd.DataFrame({'x':np.linspace(xmin113,xmax113,npoints)})
def stats2(XSA,YSA,xsel_in):
    print('Calculating stats for method 2')
    data_inp = pd.DataFrame({'x':XSA,'y':YSA})
    LW2 = lw_model.compute_group(data_inp,xsel_in,n=len(xsel_in),
            fullrange=True,method='loess',method_args={},span=0.3,se=True,level=0.95)
    
    return LW2

# prob2age produces the median, mean and mode age from a given distribution
def prob2age(age_table,depth,campaign_i):
    if campaign_i==1:
        age_table = age_table['NEEM09']
        samp_year = 2009.5
    elif campaign_i==0:
        age_table = age_table['NEEM08']
        samp_year = 2008.5
    cpd = np.cumsum(age_table[depth]) # cumulative probability distribution for given depth
    ind_med = np.abs(cpd-0.5).idxmin()
    if cpd[ind_med]>=0.5:
        med_age = age_table['age'][ind_med]
    else:
        ind_med += 1
        med_age = age_table['age'][ind_med]
    mean_age = np.sum(age_table[depth] * age_table['age'])
    mode_age = age_table['age'][age_table[depth].idxmax()]
    
    return samp_year - med_age, samp_year - mean_age, samp_year - mode_age

# plot2 plots the loess with 2sigma confidence intervals
def plot2(stats,handle,colour,line,label,Nitr):
    handle.plot(stats['x'],stats['y'],colour+line,label=label)
    handle.fill_between(stats['x'], 
                        stats['y'] - 1.96*stats['se']*np.sqrt(Nitr), 
                        stats['y'] + 1.96*stats['se']*np.sqrt(Nitr),
                        color = colour, alpha = 0.3)
    

##############################################################################
##############################################################################
# Load in data
##############################################################################
##############################################################################

firn_data_us = pd.read_excel(data_sheets1,sheet_name='Firn_summary',\
                         index_col=None,header=1)
firn_data_zw = pd.read_excel(data_sheets1,sheet_name='Zuiderweg_et_al_2013',\
                         index_col=None,header=2)

age11 = load_ages(11)
age12 = load_ages(12)
age113 = load_ages(113)

##############################################################################
##############################################################################
# Tidy up data
##############################################################################
##############################################################################

campaign = []
for ix in firn_data_us.index:
    campaign = np.append(campaign,('2009' in firn_data_us['ID'][ix])*1)
    
cut113 = np.isnan(firn_data_us['del(102/101).2']).astype(bool)

CFC11 = pd.DataFrame({'ID':firn_data_us['ID'],\
                      'campaign':campaign,\
                      'depth':firn_data_us['Depth / m'].round(1).astype(str),\
                      'del_firn':firn_data_us['del(102/101)'],\
                      'del_trop':firn_data_us['del_T(102/101)'],\
                      'age':np.ones_like(firn_data_us['ID'])})    

CFC12 = pd.DataFrame({'ID':firn_data_us['ID'],\
                      'campaign':campaign,\
                      'depth':firn_data_us['Depth / m'].round(1).astype(str),\
                      'del_firn':firn_data_us['del(102/101).1'],\
                      'del_firn_vpdb':firn_data_us['del(102/101) VPDB'],\
                      'del_trop':firn_data_us['del_T(102/101).1'],\
                      'del_trop_vpdb':firn_data_us['del_T(102/101) VPDB'],\
                      'age':np.ones_like(firn_data_us['ID'])})    

CFC113 = pd.DataFrame({'ID':firn_data_us['ID'][~cut113],\
                      'campaign':campaign[~cut113],\
                      'depth':firn_data_us['Depth / m'][~cut113].round(1).astype(str),\
                      'del_firn':firn_data_us['del(102/101).2'][~cut113],\
                      'del_trop':firn_data_us['del_T(102/101).2'][~cut113],\
                      'age':np.ones_like(firn_data_us['ID'][~cut113])})    

for icfc in [11,12,113]:
    if icfc==11:
        for ir in CFC11.index:
            CFC11['age'][ir] = prob2age(age11,CFC11['depth'][ir],CFC11['campaign'][ir])[age_def]
    elif icfc==12:
        for ir in CFC12.index:
            CFC12['age'][ir] = prob2age(age12,CFC12['depth'][ir],CFC12['campaign'][ir])[age_def]
    elif icfc==113:
        for ir in CFC113.index:
            CFC113['age'][ir] = prob2age(age113,CFC113['depth'][ir],CFC113['campaign'][ir])[age_def]
 
##############################################################################
##############################################################################
# Build loess
##############################################################################
##############################################################################
   
XSA11 = []
YSA11 = []
XSA12 = []
YSA12 = []
XSA113 = []
YSA113 = []
XSA12V = []
YSA12V = []
XSA11T = []
YSA11T = []
XSA12T = []
YSA12T = []
XSA113T = []
YSA113T = []
XSA12TV = []
YSA12TV = []

for itr in range(Nitr):
    if np.mod(itr,100)==0:
        print('Done iteration '+str(itr)+' of '+str(Nitr))
    XSA11, YSA11        = method2(11,  0, 0, XSA11,  YSA11)
    XSA12, YSA12        = method2(12,  0, 0, XSA12,  YSA12)
    XSA113, YSA113      = method2(113, 0, 0, XSA113, YSA113)
    XSA12V, YSA12V      = method2(12,  0, 1, XSA12V,  YSA12V)
    XSA11T, YSA11T      = method2(11,  1, 0, XSA11T, YSA11T)
    XSA12T, YSA12T      = method2(12,  1, 0, XSA12T, YSA12T)
    XSA113T, YSA113T    = method2(113, 1, 0, XSA113T,YSA113T)
    XSA12TV, YSA12TV    = method2(12,  1, 1, XSA12TV,YSA12TV)

print('Done iteration '+str(Nitr)+' of '+str(Nitr))

stats11 = stats2(XSA11,YSA11,xsel11)
stats12 = stats2(XSA12,YSA12,xsel12)
stats113 = stats2(XSA113,YSA113,xsel113)
stats12V = stats2(XSA12V,YSA12V,xsel12)
stats11T = stats2(XSA11T,YSA11T,xsel11)
stats12T = stats2(XSA12T,YSA12T,xsel12)
stats113T = stats2(XSA113T,YSA113T,xsel113)
stats12TV = stats2(XSA12TV,YSA12TV,xsel12)

print('Made method 2 loess from '+str(len(XSA11))+' data pairs')

fig1, ax1 = plt.subplots(1,1,figsize=(5,5),dpi=600)
plot2(stats11,ax1,'r','-','',Nitr)
plot2(stats12,ax1,'b','-','',Nitr)
plot2(stats113,ax1,'g','-','',Nitr)
    
plt.plot(CFC12['age'],CFC12['del_firn'],'rx',alpha=0.5)
plt.plot(CFC11['age'],CFC11['del_firn'],'b+',alpha=0.5)
plt.plot(CFC113['age'],CFC113['del_firn'],'gs',alpha=0.5)

    
plt.show()
    
        
##############################################################################
##############################################################################
# Plot
##############################################################################
##############################################################################

xlims = [1950,2010]  

for ir in firn_data_zw.index:
    firn_data_zw['age'][ir] = np.random.choice(CFC12['age'][CFC12['depth'].astype(float)==firn_data_zw['depth'][ir]],1)

fig1, (ax1, ax2, ax3) = plt.subplots(nrows=3,ncols=1,figsize=(19/2.2,25/2.2),dpi=600, gridspec_kw={'height_ratios': [1,2,1]})

ax1.plot(CFC11['age'][CFC11['campaign']==1],CFC11['del_firn'][CFC11['campaign']==1],'k+')
ax1.plot(CFC11['age'][CFC11['campaign']==0],CFC11['del_firn'][CFC11['campaign']==0],'kx')
plot2(stats11,ax1,'k','-','',Nitr)
ax1.set_xlim(xlims)
ax1.legend(title='CFC-11')

ax2.plot(CFC12['age'][CFC12['campaign']==1],CFC12['del_firn_vpdb'][CFC12['campaign']==1],'k+',label='NEEM 2009, this study')
ax2.plot(CFC12['age'][CFC12['campaign']==0],CFC12['del_firn_vpdb'][CFC12['campaign']==0],'kx',label='NEEM 2008')
ax2.plot(firn_data_zw['age'],firn_data_zw['Measurement1'],'r+',label='NEEM 2009, Zuiderweg et al., (2013)')
ax2.plot(firn_data_zw['age'],firn_data_zw['Measurement2'],'r+')
plot2(stats12V,ax2,'k','-','',Nitr)
ax2.set_xlim(xlims)
ax2.legend(title='CFC-12')

ax3.plot(CFC113['age'][CFC113['campaign']==1],CFC113['del_firn'][CFC113['campaign']==1],'k+')
ax3.plot(CFC113['age'][CFC113['campaign']==0],CFC113['del_firn'][CFC113['campaign']==0],'kx')
plot2(stats113,ax3,'k','-','',Nitr)
ax3.set_xlim(xlims)
ax3.legend(title='CFC-113')



##############################################################################
##############################################################################
# write csvs for loess and compounds with ages
##############################################################################
##############################################################################

CFC11.to_csv('CFC11_'+str(Nitr)+'.csv')
CFC12.to_csv('CFC12_'+str(Nitr)+'.csv')
CFC113.to_csv('CFC113_'+str(Nitr)+'.csv')
stats11.to_csv('CFCStats11_firn_'+str(Nitr)+'.csv')
stats12.to_csv('CFCStats12_firn_'+str(Nitr)+'.csv')
stats113.to_csv('CFCStats113_firn_'+str(Nitr)+'.csv')
stats11T.to_csv('CFCStats11_trop_'+str(Nitr)+'.csv')
stats12T.to_csv('CFCStats12_trop_'+str(Nitr)+'.csv')
stats113T.to_csv('CFCStats113_trop_'+str(Nitr)+'.csv')
stats12V.to_csv('CFCStats12_firn_VPDB_'+str(Nitr)+'.csv')
stats12TV.to_csv('CFCStats12_trop_VPDB_'+str(Nitr)+'.csv')
firn_data_zw.to_csv('CFCZ13.csv')


































